package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.factory.ProjectGuardFactory;
import com.hz.pm.api.common.statemachine.factory.ProjectGuardFactory.ExpertReviewToFailed;
import com.hz.pm.api.common.statemachine.factory.ProjectGuardFactory.ExpertReviewToModify;
import com.hz.pm.api.common.statemachine.factory.ProjectGuardFactory.ProjectPurchaseToAcceptGuard;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * 项目申报状态机
 * 市本级
 *
 * @author CMM
 * @since 2023/02/07 15:56
 */
@Slf4j
@Component
@RequiredArgsConstructor
@EnableStateMachine(name = ProjectStateMachineBuilderImpl.MACHINE_ID)
public class ProjectStateMachineBuilderImpl implements BaseStateMachineBuilder<Project, ProjectStateChangeEvent, ProjectStatus> {

    public static final String MACHINE_ID = "projectDeclareStateMachine";

    private final BeanFactory beanFactory;
    private final ProjectPurchaseToAcceptGuard projectPurchaseToAcceptGuard;
    private final ProjectGuardFactory.ProjectPurchaseToAdaptGuard projectPurchaseToAdaptGuard;

    @Override
    public StateMachine<ProjectStatus, ProjectStateChangeEvent> build() throws StateMachineException {
        StateMachine<ProjectStatus, ProjectStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：{}", stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    /**
     * 构建状态机
     *
     * @param beanFactory \
     */
    @Override
    public StateMachine<ProjectStatus, ProjectStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<ProjectStatus, ProjectStateChangeEvent> buildStateMachine(BeanFactory beanFactory) throws Exception {
        StateMachineBuilder.Builder<ProjectStatus, ProjectStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(beanFactory);

        builder.configureStates()
                .withStates()
                .initial(ProjectStatus.TO_BE_DECLARED)
                .states(EnumSet.allOf(ProjectStatus.class));

        builder.configureTransitions()
                // 项目立项
                .withExternal()
                .source(ProjectStatus.TO_BE_DECLARED)
                .target(ProjectStatus.ON_COMPLIANCE_REVIEW)
                .event(ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT).and()
                // 项目立项重新提交
                .withExternal()
                .source(ProjectStatus.COMPLIANCE_REVIEW_FAILED)
                .target(ProjectStatus.ON_COMPLIANCE_REVIEW)
                .event(ProjectStateChangeEvent.COMPLIANCE_REVIEW_RESUBMIT).and()
                // 合规性审查不通过
                .withExternal()
                .source(ProjectStatus.ON_COMPLIANCE_REVIEW)
                .target(ProjectStatus.COMPLIANCE_REVIEW_FAILED)
                .event(ProjectStateChangeEvent.COMPLIANCE_REVIEW_FAILED).and()
                // 合规性审查通过
                .withExternal()
                .source(ProjectStatus.ON_COMPLIANCE_REVIEW)
                .target(ProjectStatus.WITHOUT_EXPERT_REVIEW)
                .event(ProjectStateChangeEvent.COMPLIANCE_REVIEW_PASS).and()
                // 提交专家评审
                .withExternal()
                .source(ProjectStatus.WITHOUT_EXPERT_REVIEW)
                .target(ProjectStatus.ON_EXPERT_REVIEW)
                .event(ProjectStateChangeEvent.EXPERT_REVIEW_SUBMIT).and()
                // 专家评审撤回
                .withExternal()
                .source(ProjectStatus.ON_EXPERT_REVIEW)
                .target(ProjectStatus.WITHOUT_EXPERT_REVIEW)
                .event(ProjectStateChangeEvent.EXPERT_REVIEW_WITHDRAW).and()
                // 重新提交专家评审
                .withExternal()
                .source(ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY)
                .target(ProjectStatus.WITHOUT_EXPERT_REVIEW)
                .event(ProjectStateChangeEvent.EXPERT_REVIEW_RESUBMIT).and()
                // 专家评审不通过
                .withExternal()
                .source(ProjectStatus.ON_EXPERT_REVIEW)
                .target(ProjectStatus.EXPERT_REVIEW_FAILED)
                .event(ProjectStateChangeEvent.EXPERT_REVIEW_FAILED)
                .guard(new ExpertReviewToFailed())
                .and()
                .withExternal()
                .source(ProjectStatus.ON_EXPERT_REVIEW)
                .target(ProjectStatus.EXPERT_REVIEW_AFTER_MODIFY)
                .event(ProjectStateChangeEvent.EXPERT_REVIEW_FAILED)
                .guard(new ExpertReviewToModify())
                .and()
                // 项目评审通过
                .withExternal()
                .source(ProjectStatus.ON_EXPERT_REVIEW)
                .target(ProjectStatus.WITHOUT_PROJECT_REVIEW)
                .event(ProjectStateChangeEvent.EXPERT_REVIEW_PASS).and()
                // 提交项目评审
                .withExternal()
                .source(ProjectStatus.WITHOUT_PROJECT_REVIEW)
                .target(ProjectStatus.ON_PROJECT_REVIEW)
                .event(ProjectStateChangeEvent.PROJECT_REVIEW_SUBMIT).and()
                // 重新提交项目评审
                .withExternal()
                .source(ProjectStatus.PROJECT_REVIEW_FAILED)
                .target(ProjectStatus.ON_PROJECT_REVIEW)
                .event(ProjectStateChangeEvent.PROJECT_REVIEW_RESUBMIT).and()
                // 项目评审不通过
                .withExternal()
                .source(ProjectStatus.ON_PROJECT_REVIEW)
                .target(ProjectStatus.PROJECT_REVIEW_FAILED)
                .event(ProjectStateChangeEvent.PROJECT_REVIEW_FAILED).and()
                // 项目评审通过
                .withExternal()
                .source(ProjectStatus.ON_PROJECT_REVIEW)
                .target(ProjectStatus.ON_ANNUAL_PLAN)
                .event(ProjectStateChangeEvent.PROJECT_REVIEW_PASS).and()
                // 年度计划通过
                .withExternal()
                .source(ProjectStatus.ON_ANNUAL_PLAN)
                .target(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD)
                .event(ProjectStateChangeEvent.ANNUAL_PLAN_PASS).and()
                // 提交立项备案
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD)
                .target(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT).and()
                // 立项备案重新提交
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_RECORD_FAILED)
                .target(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_RESUBMIT).and()
                // 立项备案审核通过
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .target(ProjectStatus.TO_BE_PURCHASED)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_PASS).and()
                // 立项备案审核不通过
                .withExternal()
                .source(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING)
                .target(ProjectStatus.DECLARED_APPROVED_RECORD_FAILED)
                .event(ProjectStateChangeEvent.DECLARED_RECORD_FAILED).and()
                // 待采购采购备案，从待采购到实施中
                .withExternal()
                .source(ProjectStatus.TO_BE_PURCHASED)
                .target(ProjectStatus.ON_PURCHASING)
                .event(ProjectStateChangeEvent.SUBMIT_PURCHASE_NOTICE).and()
                // 填写试试计划 -> 待初验
                .withExternal()
                .source(ProjectStatus.ON_PURCHASING)
                .target(ProjectStatus.ON_ADAPTING)
                .event(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD)
                .guard(projectPurchaseToAdaptGuard)
                .and()
                // 填写试试计划 -> 待初验
                .withExternal()
                .source(ProjectStatus.ON_PURCHASING)
                .target(ProjectStatus.ACCEPTED)
                .event(ProjectStateChangeEvent.SUBMIT_PURCHASE_CONTRACT_RECORD)
                .guard(projectPurchaseToAcceptGuard)
                .and()
                // 填写试试计划 -> 待初验
                .withExternal()
                .source(ProjectStatus.ON_ADAPTING)
                .target(ProjectStatus.TO_BE_FIRST_INSPECTED)
                .event(ProjectStateChangeEvent.SUBMIT_PURCHASE_ORG_CONFIRM)
                .and()
                // 上传初验材料
                .withExternal()
                .source(ProjectStatus.TO_BE_FIRST_INSPECTED)
                .target(ProjectStatus.ON_PILOT_RUNNING)
                .event(ProjectStateChangeEvent.SUBMIT_FIRST_INSPECTED_FILES)
                .and()
                // 待终验终验申请，从待终验到终验审核中
                .withExternal()
                .source(ProjectStatus.ON_PILOT_RUNNING)
                .target(ProjectStatus.ON_FINALLY_INSPECTED)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_APPLICATION)
                .and()
                // 终审审核通过，从终审审核中到已归档
                .withExternal()
                .source(ProjectStatus.ON_FINALLY_INSPECTED)
                .target(ProjectStatus.ARCHIVED)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_PASS)
                .and()
                // 终审审核不通过，从终审审核中到终审审核不通过
                .withExternal()
                .source(ProjectStatus.ON_FINALLY_INSPECTED)
                .target(ProjectStatus.FINALLY_INSPECTED_FAILED)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_REJECT).and()
                // 终审审核中撤回，从终审审核中到待终验
                .withExternal()
                .source(ProjectStatus.ON_FINALLY_INSPECTED)
                .target(ProjectStatus.ON_PILOT_RUNNING)
                .event(ProjectStateChangeEvent.FINAL_ACCEPTANCE_IS_UNDER_REVIEW_WITHDRAW).and()
                // 终验中 重新提交
                .withExternal()
                .source(ProjectStatus.FINALLY_INSPECTED_FAILED)
                .target(ProjectStatus.ON_FINALLY_INSPECTED)
                .event(ProjectStateChangeEvent.FINAL_RE_ACCEPTANCE_APPLICATION).and();
        return builder.build();
    }

    /**
     * 持久化配置
     *
     * @author CMM
     * @since 2023/02/07 16:22
     */
    @Bean(name = "projectDeclareStatePersister")
    @Override
    public StateMachinePersister<ProjectStatus, ProjectStateChangeEvent, Project> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<ProjectStatus, ProjectStateChangeEvent, Project>() {
            @Override
            public void write(StateMachineContext<ProjectStatus, ProjectStateChangeEvent> context, Project contextObj) {
                log.info("当前项目为：{}", contextObj.getProjectCode());
            }

            @Override
            public StateMachineContext<ProjectStatus, ProjectStateChangeEvent> read(Project contextObj) {
                ProjectStatus status = ProjectStatus.getNoNull(contextObj.getStatus());
                return new DefaultStateMachineContext<>(status, null, null, null, null, MACHINE_ID);
            }
        });
    }

}
