package com.hz.pm.api.common.statemachine.util;

import com.hz.pm.api.common.model.constant.StateMachineConst;
import com.hz.pm.api.common.statemachine.builder.impl.AdaptStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.PurchaseStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderAdaptStatus;
import com.hz.pm.api.projectlib.service.IPurchaseStatusChangeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.function.Function;

/**
 * <p>
 * 标段状态机工具类
 * </p>
 *
 * @author WendyYang
 * @since 17:06 2024/2/24
 */
@Slf4j
@Component
public class AdaptStateMachineUtil implements AbstractStateMachineUtil<Purchase, AdaptStateChangeEvent> {

    //==================================================================================================================

    private final StateMachinePersister<TenderAdaptStatus, AdaptStateChangeEvent, Purchase> stateMachinePersister;
    private final StateMachine<TenderAdaptStatus, AdaptStateChangeEvent> stateMachine;
    private final IPurchaseStatusChangeService purchaseStatusChangeService;

    public AdaptStateMachineUtil(AdaptStateMachineBuilderImpl adaptStateMachineBuilder,
                                 IPurchaseStatusChangeService purchaseStatusChangeService) {
        this.purchaseStatusChangeService = purchaseStatusChangeService;
        this.stateMachine = adaptStateMachineBuilder.build();
        this.stateMachinePersister = adaptStateMachineBuilder.stateMachinePersister();
    }

    @Override
    public Function<Purchase, Integer> statusFunction() {
        return Purchase::getAdaptStatus;
    }

    @Override
    public <T> void execute(Purchase purchase, AdaptStateChangeEvent event, T extraArgs) throws Exception {
        PurchaseStatusChange change = new PurchaseStatusChange();
        change.setBeforeStatus(purchase.getAdaptStatus());
        change.setEvent(event.name());
        change.setBidId(purchase.getId());
        change.setProjectId(purchase.getProjectId());
        change.setProjectCode(ProjectIdCodeCacheUtil.get(purchase.getProjectId()));
        change.setCreateOn(LocalDateTime.now());
        Message<AdaptStateChangeEvent> message = MessageBuilder.withPayload(event)
                .setHeader(StateMachineConst.MessageHeader.PURCHASE, purchase)
                .setHeader(StateMachineConst.MessageHeader.EXTRA_ARGS, extraArgs)
                .build();
        //初始化状态机
        stateMachinePersister.restore(stateMachine, purchase);
        stateMachine.sendEvent(message);
        change.setAfterStatus(purchase.getAdaptStatus());
        purchaseStatusChangeService.save(change);
        log.info("调用状态机后的标段状态为：{} ==> {}", change.getBeforeStatus(), change.getAfterStatus());
    }

}
