package com.hz.pm.api.external;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.NumberUtil;
import com.hz.pm.api.common.enumeration.FileOrigin;
import com.hz.pm.api.common.util.PreviewFileIdEncryptUtil;
import com.hz.pm.api.external.model.PreviewFileIdBO;
import com.hz.pm.api.external.model.dto.MhFileInfoDTO;
import com.hz.pm.api.meeting.entity.config.WebProps;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.service.FileService;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.Duration;
import java.util.List;

/**
 * <p>
 * FilePreviewClient
 * </p>
 *
 * @author WendyYang
 * @since 09:27 2024/3/18
 */
@Component
@RequiredArgsConstructor
public class FilePreviewClient {

    private static final Logger log = LoggerFactory.getLogger(FilePreviewClient.class);

    @Value("${mh.file-preview.public-host:}")
    private String mhFilePreviewPublicUrl;

    @Value("${mh.file-preview.private-host:}")
    private String mhFilePreviewPrivateUrl;

    private final FileService fileService;
    private final MhFileClient mhFileClient;

    private static final String FILE_DOWN_NO_AUTH_URL = "/api/v1/file/preview/anonymous/down/";

    //==================================================================================================================

    private static final String PREVIEW_PATH_FMT = "%s/public/risen/core/resrc/view/pdf_view.html.do?strMap.remote_url=%s&strMap.ext=%s";

    public String getFilePreviewUrl(String fileId, HttpServletRequest request) {
        String serverName = request.getServerName();
        log.info("serverName:{}", serverName);
        StringBuilder fileDownUrl = new StringBuilder();
        String filePreviewUrlHost;
        if (serverName.contains("weixin") || serverName.contains("10.54.38.13")) {
            // 浙政钉访问需要转换为互联网可访问的地址
            fileDownUrl.append(WebProps.zzdApiUrl);
            filePreviewUrlHost = mhFilePreviewPublicUrl;
        } else {
            fileDownUrl.append(WebProps.apiUrl);
            filePreviewUrlHost = mhFilePreviewPrivateUrl;
        }
        String fileSuffix;
        // 信产文件ID为字符串、当前系统文件ID为数字
        if (!NumberUtil.isLong(fileId)) {
            List<MhFileInfoDTO> fileInfos = mhFileClient.listFileInfo(fileId);
            MhFileInfoDTO mhFileInfo = fileInfos.get(0);
            fileSuffix = mhFileInfo.getFileSuffix();
            String tmpFileId = generateFileId(fileId, FileOrigin.MH);
            fileDownUrl.append(FILE_DOWN_NO_AUTH_URL).append(tmpFileId);
        } else {
            String tmpFileId = generateFileId(fileId, FileOrigin.OSS);
            fileDownUrl.append(FILE_DOWN_NO_AUTH_URL).append(tmpFileId);
            File file = fileService.getById(fileId);
            fileSuffix = file.getSuffix();
        }
        return String.format(PREVIEW_PATH_FMT, filePreviewUrlHost, fileDownUrl, fileSuffix);
    }

    private static String generateFileId(String fileId, FileOrigin oss) {
        PreviewFileIdBO previewFileId = PreviewFileIdBO.builder()
                // 默认过期时间6小时
                .expireTime(System.currentTimeMillis() + Duration.ofHours(6).toMillis())
                .fileId(fileId)
                .fileOrigin(oss)
                .build();
        return PreviewFileIdEncryptUtil.encryptFileId(previewFileId.concat());
    }

    private static PreviewFileIdBO parseFileId(String fileId) {
        String decryptFileId = PreviewFileIdEncryptUtil.decryptFileId(fileId);
        String[] split = decryptFileId.split("#");
        if (ArrayUtil.isEmpty(split) || split.length != 3) {
            throw BizException.wrap("文件ID无效");
        }
        return PreviewFileIdBO.builder()
                .fileId(split[0])
                .expireTime(Long.parseLong(split[1]))
                .fileOrigin(FileOrigin.valueOf(split[2]))
                .build();
    }

    public void downloadFile(String fileId, HttpServletResponse response) {
        try {
            PreviewFileIdBO previewFileId = parseFileId(fileId);
            if (previewFileId.getExpireTime() != null
                    && previewFileId.getExpireTime() > 0
                    && System.currentTimeMillis() > previewFileId.getExpireTime()) {
                throw BizException.wrap("文件地址已过期，请重试");
            }
            if (FileOrigin.MH.equals(previewFileId.getFileOrigin())) {
                mhFileClient.download(previewFileId.getFileId(), response);
            } else if (FileOrigin.OSS.equals(previewFileId.getFileOrigin())) {
                fileService.download(Long.valueOf(previewFileId.getFileId()), response);
            } else {
                throw BizException.wrap("文件类型不支持");
            }
        } catch (Exception e) {
            log.info("文件下载失败：{}", fileId, e);
            throw BizException.wrap("文件下载失败：%s", fileId);
        }
    }

}
