package com.hz.pm.api.external;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.resource.ClassPathResource;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.external.model.dto.MhFileInfoDTO;
import com.hz.pm.api.external.model.dto.MhRetDTO;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * MhFileClient
 * </p>
 *
 * @author WendyYang
 * @since 20:24 2023/12/28
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class MhFileClient {

    private final Environment environment;

    private static final int TIMEOUT_MILLS = 10000;

    @Value("${mh.file.down-url:}")
    private String fileDownUrl;

    @Value("${mh.file.detail-url:}")
    private String fileDetailUrl;

    @Value("${mh.file.upload-url:}")
    private String fileUploadUrl;

    private boolean isProdEnv() {
        String[] profiles = environment.getActiveProfiles();
        return Arrays.asList(profiles).contains("prod");
    }

    public void download(String fileId, HttpServletResponse response) {
        String fileUrl = fileDownUrl + "/" + fileId;
        try (ServletOutputStream stream = response.getOutputStream()) {
            long size = HttpUtil.download(fileUrl, stream, true);
            log.info("下载文件：{}，大小为：{}", fileId, size);
        } catch (Exception e) {
            log.error("信创平台文件下载异常：{}", fileId, e);
            if (isProdEnv()) {
                throw BizException.wrap("下载文件失败");
            }
        }
    }

    public File downloadToTmpFile(String fileId) {
        String fileUrl = fileDownUrl + "/" + fileId;
        try {
            List<MhFileInfoDTO> files = listFileInfo(fileId);
            MhFileInfoDTO file = files.get(0);
            String tmpDirPath = StrUtil.appendIfMissing(FileUtil.getTmpDirPath(), "/");
            String tmpFilePath = tmpDirPath + file.getFileName() + "." + file.getFileSuffix();
            File tmpFile = new File(tmpFilePath);
            long size = HttpUtil.downloadFile(fileUrl, tmpFile);
            log.info("下载文件：{}，大小为：{}", fileId, size);
            return tmpFile;
        } catch (Exception e) {
            log.error("信创平台文件下载异常：{}", fileId, e);
            if (isProdEnv()) {
                throw BizException.wrap("下载文件失败");
            }
            ClassPathResource resource = new ClassPathResource("/response/ret-xcfhx-report-file.pdf");
            File destFile = new File(FileUtil.getTmpDirPath() + RandomUtil.randomString(20) + ".pdf");
            return FileUtil.writeFromStream(resource.getStream(), destFile, true);
        }
    }

    public List<MhFileInfoDTO> listFileInfo(String fileIds) {
        String url = fileDetailUrl + "?ids=" + fileIds;
        try {
            String body = HttpUtil.get(url, TIMEOUT_MILLS);
            MhRetDTO<List<MhFileInfoDTO>> ret = JSONUtil.toBean(body,
                    new TypeReference<MhRetDTO<List<MhFileInfoDTO>>>() {
                    }, false);
            if (ret.isOk()) {
                return ret.getData();
            }
            throw BizException.wrap(ret.getMsg());
        } catch (Exception e) {
            log.error("信创平台文件详情获取异常：{}", fileIds, e);
            if (isProdEnv()) {
                throw BizException.wrap("获取文件详情失败");
            } else {
                return Collections.emptyList();
            }
        }
    }

    public String uploadFile(File file) {
        HttpRequest request = HttpUtil.createPost(fileUploadUrl);
        request.form("file", file);
        try (HttpResponse response = request.execute()) {
            MhRetDTO retObj = JSONUtil.toBean(response.body(), MhRetDTO.class);
            if (retObj.isOk()) {
                return retObj.getData().toString();
            }
            log.error("文件上传失败：{}", retObj);
            throw BizException.wrap("文件上传失败");
        }
    }

}
