package com.hz.pm.api.open.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.util.BizDateUtil;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.open.model.po.MhSyncProjectListPO;
import com.hz.pm.api.open.model.po.TenderListReq;
import com.hz.pm.api.open.model.vo.DeclaredProjectInfoToMhVO;
import com.hz.pm.api.open.model.vo.ReferSystemReplaceInfoToMhVO;
import com.hz.pm.api.open.model.vo.TenderToMhDetailVO;
import com.hz.pm.api.open.util.OpenApiCheckSignUtil;
import com.hz.pm.api.projectdeclared.model.entity.Contract;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.service.IContractService;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.entity.MhSystemReplaceInfo;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.service.IMhSystemReplaceInfoService;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.file.entity.vo.result.AttachFileVo;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import com.ningdatech.file.service.FileService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * OpenApiProjectLibManage
 * </p>
 *
 * @author WendyYang
 * @since 10:22 2024/11/7
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class OpenApiProjectLibManage {

    private final FileService fileService;
    private final OpenApiCheckSignUtil openApiCheckSignUtil;
    private final IProjectService projectService;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IMhSystemReplaceInfoService mhSystemReplaceInfoService;
    private final IPurchaseService purchaseService;
    private final IContractService contractService;
    private final IOperationService operationService;

    /**
     * 获取已复核通过的项目列表
     *
     * @param po 参数
     * @return 复核通过的项目
     */
    public List<DeclaredProjectInfoToMhVO> listDeclaredProjects(MhSyncProjectListPO po) {
        openApiCheckSignUtil.checkSign(po.getKey());
        StringBuilder sql = new StringBuilder(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE)
                .append(StrUtil.format(" and npsc.event = '{}'", ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT));
        if (po.getCreateOn() != null) {
            sql.append(String.format(" and npsc.create_on >= '%s'", BizDateUtil.toDateStr(po.getCreateOn())));
        }
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode, Project::getProjectName, Project::getBuildOrgCode,
                        Project::getStatus, Project::getReviewAmount, Project::getDeclareAmount, Project::getProjectYear,
                        Project::getCreateOn, Project::getBuildOrgName)
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE)
                .exists(sql.toString())
                .eq(po.getProjectYear() != null, Project::getProjectYear, po.getProjectYear())
                .orderByDesc(Project::getCreateOn);
        List<Project> projects = projectService.list(query);
        if (projects.isEmpty()) {
            return Collections.emptyList();
        }
        List<String> projectCodes = CollUtils.fieldList(projects, Project::getProjectCode);
        List<ProjectGovSystemReplaceInfos> replaceInfos = systemReplaceInfosService.listByProjectCodes(projectCodes);
        Map<String, List<ProjectGovSystemReplaceInfos>> originReplaceInfoMap;
        Map<String, List<MhSystemReplaceInfo>> replaceInfoMap;
        if (!replaceInfos.isEmpty()) {
            originReplaceInfoMap = CollUtils.group(replaceInfos, ProjectGovSystemReplaceInfos::getProjectCode);
            List<String> sourceSystemIds = CollUtils.fieldList(replaceInfos, ProjectGovSystemReplaceInfos::getSourceSystemId);
            List<MhSystemReplaceInfo> mhSystemReplaceInfos = mhSystemReplaceInfoService.listByIds(sourceSystemIds);
            Map<String, MhSystemReplaceInfo> mhReplaceInfoMap = CollUtils.listToMap(mhSystemReplaceInfos, MhSystemReplaceInfo::getId);
            replaceInfoMap = replaceInfos.stream()
                    .filter(w -> mhReplaceInfoMap.containsKey(w.getSourceSystemId()))
                    .collect(Collectors.groupingBy(ProjectGovSystemReplaceInfos::getProjectCode,
                            Collectors.mapping(w -> mhReplaceInfoMap.get(w.getSourceSystemId()), Collectors.toList())));
        } else {
            originReplaceInfoMap = Collections.emptyMap();
            replaceInfoMap = Collections.emptyMap();
        }

        return projects.stream().map(w -> {
            DeclaredProjectInfoToMhVO item = new DeclaredProjectInfoToMhVO();
            item.setId(w.getId());
            item.setStatus(w.getStatus());
            item.setCreateOn(w.getCreateOn());
            item.setProjectYear(w.getProjectYear());
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setBuildOrgCode(w.getBuildOrgCode());
            item.setBuildOrgName(w.getBuildOrgName());
            item.setDeclareAmount(w.getDeclareAmount());
            item.setReviewAmount(w.getReviewAmount());
            if (replaceInfoMap.containsKey(w.getProjectCode())) {
                List<MhSystemReplaceInfo> currReplaceInfos = replaceInfoMap.get(w.getProjectCode());
                List<ReferSystemReplaceInfoToMhVO> referSystemList = CollUtils.convert(currReplaceInfos,
                        t -> ReferSystemReplaceInfoToMhVO.builder()
                                .id(t.getId())
                                .buildOrgCode(t.getBuildOrgCode())
                                .buildOrgName(t.getBuildOrgName())
                                .systemName(t.getSourceName())
                                .alternativeName(t.getTargetName())
                                .alternativeType(t.getReplaceType())
                                .build());
                item.setReferSystemList(referSystemList);
            } else if (originReplaceInfoMap.containsKey(w.getProjectCode())) {
                List<ProjectGovSystemReplaceInfos> currReplaceInfos = originReplaceInfoMap.get(w.getProjectCode());
                List<ReferSystemReplaceInfoToMhVO> referSystemList = CollUtils.convert(currReplaceInfos,
                        t -> ReferSystemReplaceInfoToMhVO.builder()
                                .id(t.getSourceSystemId())
                                .buildOrgCode(Integer.valueOf(w.getBuildOrgCode()))
                                .buildOrgName(w.getBuildOrgName())
                                .systemName(t.getSourceSystem())
                                .alternativeName(t.getTargetSystem())
                                .alternativeType(getReplaceType(t))
                                .build());
                item.setReferSystemList(referSystemList);
            } else {
                item.setReferSystemList(Collections.emptyList());
            }
            return item;
        }).collect(Collectors.toList());
    }

    private static String getReplaceType(ProjectGovSystemReplaceInfos t) {
        String replaceType = StrUtil.EMPTY;
        if (NumberUtil.isNumber(t.getReplaceType())) {
            Optional<GovSystemReplaceTypeEnum> optReplaceType = GovSystemReplaceTypeEnum.get(Integer.valueOf(t.getReplaceType()));
            if (optReplaceType.isPresent()) {
                GovSystemReplaceTypeEnum replaceTypeEnum = optReplaceType.get();
                replaceType = Optional.ofNullable(replaceTypeEnum.getMhReplaceType())
                        .flatMap(mhReplaceType -> Optional.of(mhReplaceType.getCode()))
                        .orElse(replaceTypeEnum.getVal());
            }
        }
        return replaceType;
    }

    private String collectFileStrIds(String fileCtx, List<Long> fileIds, Map<Long, String> fileIdMap) {
        if (StrUtil.isNotBlank(fileCtx)) {
            List<FileResultVO> fileRetList;
            try {
                fileRetList = JSONUtil.toList(fileCtx, FileResultVO.class);
            } catch (Exception e) {
                log.warn("文件ID解析异常：{}", fileCtx);
                return StrUtil.EMPTY;
            }
            for (FileResultVO fileRet : fileRetList) {
                if (StrUtil.isNotBlank(fileRet.getStrId())) {
                    fileIdMap.put(fileRet.getId(), fileRet.getStrId());
                } else {
                    fileIds.add(fileRet.getId());
                }
            }
            return CollUtils.joinByComma(fileRetList, FileResultVO::getId);
        }
        return StrUtil.EMPTY;
    }

    private String convertToFileStrId(String content, Map<Long, String> fileIdMap) {
        List<Long> fileIds = BizUtils.splitToLong(content);
        if (CollUtil.isNotEmpty(fileIds)) {
            return CollUtils.joinByComma(fileIds, fileIdMap::get);
        }
        return StrUtil.EMPTY;
    }

    public List<TenderToMhDetailVO> listTenders(TenderListReq req) {
        openApiCheckSignUtil.checkSign(req.getKey());
        Wrapper<Project> projQuery = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode)
                .eq(req.getProjectYear() != null, Project::getProjectYear, req.getProjectYear())
                .eq(req.getBuildOrgCode() != null, Project::getBuildOrgCode, req.getBuildOrgCode())
                .eq(req.getProjectCode() != null, Project::getProjectCode, req.getProjectCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED)
                .eq(Project::getNewest, Boolean.TRUE);
        List<Project> projects = projectService.list(projQuery);
        if (projects.isEmpty()) {
            return Collections.emptyList();
        }
        List<Long> projectIds = projects.stream()
                .map(w -> ProjectIdCodeCacheUtil.get(w.getProjectCode()))
                .flatMap(Collection::stream)
                .collect(Collectors.toList());
        Wrapper<Purchase> purQuery = Wrappers.lambdaQuery(Purchase.class)
                .in(Purchase::getProjectId, projectIds)
                .ge(req.getBidUpdateOn() != null, Purchase::getUpdateOn, req.getBidUpdateOn())
                .orderByDesc(Purchase::getCreateOn)
                .orderByDesc(Purchase::getId);
        List<Purchase> purchases = purchaseService.list(purQuery);
        if (purchases.isEmpty()) {
            return Collections.emptyList();
        }
        List<Long> bidIds = CollUtils.fieldList(purchases, Purchase::getId);
        List<ProjectGovSystemReplaceInfos> systemReplaceInfos = systemReplaceInfosService.listByBidIds(bidIds);
        Map<Long, List<String>> bidIdSystemIdMap = systemReplaceInfos.stream()
                .filter(w -> StrUtil.isNotBlank(w.getSourceSystemId()))
                .collect(Collectors.groupingBy(ProjectGovSystemReplaceInfos::getBidId,
                        Collectors.mapping(ProjectGovSystemReplaceInfos::getSourceSystemId, Collectors.toList())));
        // 合同信息
        Map<Long, Contract> contractMap = contractService.listByBidIds(bidIds);
        // 实施信息
        List<Operation> operations = operationService.listByBidIds(bidIds);
        Map<Long, Operation> operationMap = CollUtils.listToMap(operations, Operation::getBidId);

        List<Long> fileIds = new ArrayList<>();
        Map<Long, String> fileIdMap = new HashMap<>();
        List<TenderToMhDetailVO> collect = purchases.stream().map(w -> {
            TenderToMhDetailVO tender = new TenderToMhDetailVO();
            tender.setBidId(w.getId());
            tender.setCreateOn(w.getCreateOn());
            tender.setProjectCode(ProjectIdCodeCacheUtil.get(w.getProjectId()));
            tender.setName(w.getBidName());
            tender.setBuyMain(String.valueOf(w.getPurchaseUnitId()));
            tender.setBuildContent(w.getBidBuildContent());
            tender.setBudgetPrice(w.getBidBudget());
            tender.setBuyType(w.getPurchaseMethod());
            tender.setImplCompany(w.getSupplier());
            tender.setIsFinishLp(w.getHasSecurityEval());
            tender.setIsFinishXc(w.getMatchXcfhx());
            tender.setXcReportFiles(collectFileStrIds(w.getXcfhxReportFiles(), fileIds, fileIdMap));
            tender.setLpReportFiles(collectFileStrIds(w.getSecurityEvalMaterials(), fileIds, fileIdMap));
            tender.setSystemIdList(bidIdSystemIdMap.get(w.getId()));
            tender.setContact(w.getPurchasePhone());
            tender.setContactPerson(w.getPurchaseContact());
            Operation operation = operationMap.get(w.getId());
            if (operation != null) {
                tender.setRefactorTime(operation.getActualAdaptionTime());
            }

            BizUtils.notNull(w.getXcfhxEvalFinishDate(), ft -> tender.setXcCompleteTime(ft.atStartOfDay()));
            BizUtils.notNull(w.getSecurityEvalDate(), ft -> tender.setLpCompleteTime(ft.atStartOfDay()));
            BizUtils.notNull(w.getFinalInspectionDate(), ft -> tender.setFinalTime(ft.atStartOfDay()));
            BizUtils.notNull(w.getPreliminaryInspectionDate(), ft -> tender.setInitialTime(ft.atStartOfDay()));

            tender.setTenderFiles(collectFileStrIds(w.getBidDoc(), fileIds, fileIdMap));
            tender.setInitialReportFiles(collectFileStrIds(w.getPreliminaryInspectionMaterials(), fileIds, fileIdMap));
            tender.setFinalReportFiles(collectFileStrIds(w.getFinalInspectionMaterials(), fileIds, fileIdMap));
            tender.setBidNotifyFiles(collectFileStrIds(w.getAcceptanceLetter(), fileIds, fileIdMap));

            Contract contract = contractMap.get(w.getId());
            if (contract != null) {
                tender.setBuyConFiles(collectFileStrIds(contract.getAttachment(), fileIds, fileIdMap));
                tender.setConSignTime(contract.getContractTime());
                tender.setConPrice(contract.getTotalAmount());
                tender.setImplCompanyPhone(contract.getSupplierContactInfo());
                tender.setImplCompanyPeople(contract.getSupplierContact());
            }
            return tender;
        }).collect(Collectors.toList());
        if (!fileIds.isEmpty()) {
            List<AttachFileVo> byIds = fileService.getByIds(fileIds);
            for (AttachFileVo byId : byIds) {
                fileIdMap.put(byId.getFileId(), byId.getStrFileId());
            }
        }
        for (TenderToMhDetailVO w : collect) {
            w.setXcReportFiles(convertToFileStrId(w.getXcReportFiles(), fileIdMap));
            w.setLpReportFiles(convertToFileStrId(w.getLpReportFiles(), fileIdMap));
            w.setFinalReportFiles(convertToFileStrId(w.getFinalReportFiles(), fileIdMap));
            w.setBidNotifyFiles(convertToFileStrId(w.getBidNotifyFiles(), fileIdMap));
            w.setTenderFiles(convertToFileStrId(w.getTenderFiles(), fileIdMap));
            w.setBuyConFiles(convertToFileStrId(w.getBuyConFiles(), fileIdMap));
            w.setInitialReportFiles(convertToFileStrId(w.getInitialReportFiles(), fileIdMap));
        }
        return collect;
    }

}
