package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.*;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.*;

/**
 * 项目申报处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ComplianceReviewHandle extends AbstractProcessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;
    private final IProjectInstService projectInstService;
    private final ProcessInstanceService processInstanceService;

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.COMPLIANCE_REVIEW;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        Project project = ctx.getProject();
        boolean notDeclared = ProjectStatus.TO_BE_DECLARED.eq(project.getStatus());
        // 项目申报节点
        ProcessDetailVO startNode = new ProcessDetailVO();
        startNode.setProcessName(ProcessNode.PROJECT_DECLARED.getName());
        startNode.setFinishTime(project.getCreateOn());
        startNode.setStepStatus(notDeclared ? StepStatusEnum.NOT_START : StepStatusEnum.COMPLETED);
        ctx.getProcessDetails().add(startNode);
        // 合规性审查
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        // 项目状态为待立项批复之前的状态
        Map<ProjectStateChangeEvent, ProjectStatusChange> eventMap;
        if (notDeclared) {
            eventMap = Collections.emptyMap();
        } else {
            eventMap = projectStatusChangeService.listLastEventMap(project.getProjectCode());
        }
        ctx.setProjectStatusChanges(eventMap);
        if (!eventMap.containsKey(PROJECT_APPLICATION_SUBMIT)) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            List<Long> projectIds = ProjectIdCodeCacheUtil.get(project.getProjectCode());
            List<ProjectInst> projectInstList = projectInstService.listByProjectIds(projectIds);
            Map<InstTypeEnum, ProjectInst> instMap = BizUtils.groupFirstMap(projectInstList,
                    w -> InstTypeEnum.getNoNull(w.getInstType()),
                    Comparator.comparing(ProjectInst::getCreateOn).reversed());
            ctx.setProjectInstMap(instMap);
            Optional<ProjectStatusChange> event = eventMap.values().stream()
                    .filter(w -> PROJECT_APPLICATION_SUBMIT.eq(w.getEvent())
                            || COMPLIANCE_REVIEW_FAILED.eq(w.getEvent())
                            || COMPLIANCE_REVIEW_RESUBMIT.eq(w.getEvent())
                            || COMPLIANCE_REVIEW_PASS.eq(w.getEvent())
                    ).max(Comparator.comparing(ProjectStatusChange::getCreateOn));
            if (event.isPresent()) {
                ProjectStatusChange change = event.get();
                ProjectInst inst = instMap.get(InstTypeEnum.COMPLIANCE_REVIEW);
                if (COMPLIANCE_REVIEW_PASS.eq(change.getEvent()) && inst == null) {
                    detail.setStepStatus(StepStatusEnum.COMPLETED);
                    detail.setFinishTime(change.getCreateOn());
                } else if (inst != null) {
                    ProcessProgressVo progress = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
                    buildStepStatus(detail, progress, change);
                } else {
                    detail.setStepStatus(StepStatusEnum.NOT_START);
                }
            } else {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            }
        }
        ctx.setPreStepStatus(detail.getStepStatus());
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
