package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.exception.ReturnException;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.projectlib.entity.ProjectApprovalAmount;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.constant.ProjectStatusConstant;
import com.hz.pm.api.projectlib.model.dto.ProjectApprovalAmountDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.req.SubmitAmountApprovalReq;
import com.hz.pm.api.projectlib.model.vo.AmountApprovalProgressVO;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.service.IProjectApprovalAmountService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.scheduler.listener.event.MhTodoSendEvent;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * AmountApprovalManage
 * </p>
 *
 * @author WendyYang
 * @since 17:10 2024/9/4
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class AmountApprovalManage {

    private final IProjectApprovalAmountService projectApprovalAmountService;
    private final IProjectService projectService;
    private final UserInfoHelper userInfoHelper;
    private final MhUnitCache mhUnitCache;

    private boolean projectQueryPermission(LambdaQueryWrapper<Project> query, UserFullInfoDTO user) {
        boolean queryState = true;
        Optional<DataScopeDTO> currDs = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!currDs.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            switch (currDs.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
                    break;
                case COMPANY_MANAGER:
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    List<String> viewUnitIdList = CollUtils.convert(childUnitIds, String::valueOf);
                    //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    query.and(q1 -> q1.in(Project::getBuildOrgCode, viewUnitIdList)
                            .or(q2 -> q2.eq(Project::getStage, ProjectStatus.NOT_APPROVED.getCode())
                                    .in(Project::getSuperOrgCode, viewUnitIdList)));
                    break;
                case SUPER_ADMIN:
                    //超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                    break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
        }
        return queryState;
    }

    public PageVo<ProjectLibListItemVO> page(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        if (!projectQueryPermission(query, user)) {
            return PageVo.empty();
        }
        ProjectManageUtil.projectQuery(query, req);
        ProjectManageUtil.projectBaseQuery(query);
        query.eq(Project::getAnnualPlanOpened, Boolean.TRUE)
                .eq(Project::getNewest, Boolean.TRUE)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED);
        buildAmountApprovalFilter(req.getIsAmountApproval(), query);
        Page<Project> page = projectService.page(req.page(), query);
        long total = page.getTotal();
        if (total == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setUnitStrip(w.getUnitStrip());
            item.setUnitStripName(MhUnitStripEnum.getVal(w.getUnitStrip()));
            item.setProjectYear(w.getProjectYear());
            item.setProjectType(w.getProjectType());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setBuildOrg(w.getBuildOrgName());
            item.setCreateOn(w.getCreateOn());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setReviewAmount(w.getReviewAmount());
            return item;
        });
        return PageVo.of(records, total);
    }

    private static void buildAmountApprovalFilter(Boolean isAmountApproval, LambdaQueryWrapper<Project> query) {
        if (isAmountApproval != null) {
            if (Boolean.TRUE.equals(isAmountApproval)) {
                query.gt(Project::getApprovalAmount, 0)
                        .isNotNull(Project::getApprovalAmount);
            } else {
                query.and(q -> q.isNull(Project::getApprovalAmount)
                        .or(q1 -> q1.le(Project::getApprovalAmount, 0)));
            }
        }
    }

    public AmountApprovalProgressVO amountApprovalProgress(ProjectListReq req) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.initQuery();
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        if (!projectQueryPermission(query, user)) {
            return null;
        }
        ProjectManageUtil.projectQuery(query, req);
        ProjectManageUtil.projectBaseQuery(query);
        query.eq(Project::getAnnualPlanOpened, Boolean.TRUE)
                .eq(Project::getNewest, Boolean.TRUE)
                .ne(Project::getStage, ProjectStatus.STOPPED.getCode())
                .notIn(Project::getStatus, ProjectStatusConstant.CHANGE_DELETE_PASSED);
        buildAmountApprovalFilter(req.getIsAmountApproval(), query);
        List<Project> page = projectService.list(query);
        AmountApprovalProgressVO stat = new AmountApprovalProgressVO();
        stat.setTotalCount(page.size());
        stat.setApprovalCount(CollUtil.count(page, w -> DecimalUtil.noNullAndGtZero(w.getApprovalAmount())));
        stat.setUnApprovalCount(stat.getTotalCount() - stat.getApprovalCount());
        return stat;
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitAmountApproval(SubmitAmountApprovalReq req) {
        Project project = projectService.getNewestNoNull(req.getProjectId());
        if (ProjectStatus.STOPPED.eq(project.getStage())
                || ProjectStatus.CHANGE.eq(project.getStage())) {
            throw ReturnException.wrap("暂不允许下达资金");
        }
        boolean approved = DecimalUtil.noNullAndGtZero(project.getApprovalAmount());
        BigDecimal reviewAmount = approved ? project.getReviewAmount() : req.getReviewAmount();
        BigDecimal currTotalApprovalAmount = NumberUtil.add(project.getApprovalAmount(), req.getApprovalAmount());
        if (req.getApprovalAmount() != null && NumberUtil.isGreater(currTotalApprovalAmount, reviewAmount)) {
            throw ReturnException.wrap("历年资金之和大于评审项目总投资，无法提交");
        }
        Integer nextYear = null;
        if (approved) {
            if (ObjUtil.isAllEmpty(req.getCarryOverAmount(), req.getApprovalAmount())) {
                throw ReturnException.wrap("下达资金或结转资金不能为空");
            }
            if (req.getCarryOverAmount() != null) {
                ProjectApprovalAmount last = projectApprovalAmountService.last(project.getProjectCode());
                BigDecimal lastTotalAmount = DecimalUtil.sum(last.getApprovalAmount(), last.getCarryOverAmount());
                if (NumberUtil.isGreater(req.getCarryOverAmount(), lastTotalAmount)) {
                    throw ReturnException.wrap("结转资金不允许超过上一年度总资金");
                }
                nextYear = last.getApprovalYear() + 1;
            }
        } else if (ObjUtil.hasNull(req.getReviewAmount() == null, req.getApprovalAmount())) {
            throw ReturnException.wrap("评审总投资和下达资金不能为空");
        }
        if (nextYear == null) {
            nextYear = getLastApprovalYear(project);
        }
        if (req.getApprovalAmount() != null) {
            Wrapper<Project> projectWrapper = Wrappers.lambdaUpdate(Project.class)
                    .set(StrUtil.isNotBlank(req.getApprovalAmountFile()), Project::getApprovalAmountFile, req.getApprovalAmountFile())
                    .set(Project::getApprovalAmount, currTotalApprovalAmount)
                    .set(req.getReviewAmount() != null, Project::getReviewAmount, req.getReviewAmount())
                    .set(Project::getUpdateOn, LocalDateTime.now())
                    .set(Project::getUpdateBy, LoginUserUtil.getUserId())
                    .eq(Project::getId, project.getId());
            projectService.update(projectWrapper);
        }
        ProjectApprovalAmount projectApprovalAmount = new ProjectApprovalAmount();
        projectApprovalAmount.setApprovalYear(nextYear);
        projectApprovalAmount.setApprovalAmount(req.getApprovalAmount());
        projectApprovalAmount.setProjectId(project.getId());
        projectApprovalAmount.setProjectCode(project.getProjectCode());
        projectApprovalAmount.setCarryOverAmount(req.getCarryOverAmount());
        projectApprovalAmountService.save(projectApprovalAmount);
        if (!approved) {
            SpringUtil.publishEvent(MhTodoSendEvent.of(MHTodoTypeEnum.DECLARED_RECORD_APPLY, project));
        }
    }

    private Integer getLastApprovalYear(Project project) {
        ProjectApprovalAmount last = projectApprovalAmountService.last(project.getProjectCode());
        Integer currProjectYear;
        if (last == null) {
            currProjectYear = project.getProjectYear();
        } else {
            currProjectYear = last.getApprovalYear() + 1;
        }
        return currProjectYear;
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void initOldApprovalAmount() {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .gt(Project::getApprovalAmount, 0)
                .isNotNull(Project::getApprovalAmount)
                .eq(Project::getNewest, Boolean.TRUE);
        List<Project> projects = projectService.list(query);
        List<ProjectApprovalAmount> approvalAmounts = projects.stream().map(w -> {
            ProjectApprovalAmount approvalAmount = new ProjectApprovalAmount();
            approvalAmount.setApprovalYear(w.getProjectYear());
            approvalAmount.setApprovalAmount(w.getApprovalAmount());
            approvalAmount.setProjectId(w.getId());
            approvalAmount.setProjectCode(w.getProjectCode());
            return approvalAmount;
        }).collect(Collectors.toList());
        projectApprovalAmountService.saveBatch(approvalAmounts);
    }

    public List<ProjectApprovalAmountDTO> listApprovalAmounts(String projectCode) {
        List<ProjectApprovalAmount> approvalAmounts = projectApprovalAmountService.listAsc(projectCode);
        return BeanUtil.copyToList(approvalAmounts, ProjectApprovalAmountDTO.class);
    }

}
