package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.RegionConst;
import com.hz.pm.api.common.helper.RegionCacheHelper;
import com.hz.pm.api.projectdeclared.utils.ProjectCodeGenUtil;
import com.hz.pm.api.projectlib.model.dto.ProjectCollectionSaveDTO;
import com.hz.pm.api.projectlib.model.entity.ProjectCollection;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectCollectionVO;
import com.hz.pm.api.projectlib.service.IProjectCollectionService;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname ProjectCollectionManage
 * @Description
 * @Date 2023/6/25 9:13
 * @Author PoffyZhang
 */
@Component
@Slf4j
@AllArgsConstructor
public class ProjectCollectionManage {

    private final IProjectCollectionService collectionService;

    private final RegionCacheHelper regionCacheHelper;

    private final ProjectCodeGenUtil projectCodeGenUtil;

    /**
     * 项目归集
     *
     * @param req
     * @return
     */
    public PageVo<ProjectCollectionVO> list(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.userDetail();
        String buildOrgCode = req.getBuildOrgCode();
        //如果是超管 可以看所有
        if (user.hasSuperAdmin()) {
            buildOrgCode = null;
        }

        Page<ProjectCollection> page = req.page();

        LambdaQueryWrapper<ProjectCollection> wrapper = Wrappers.lambdaQuery(ProjectCollection.class)
                .like(StringUtils.isNotBlank(req.getProjectName()), ProjectCollection::getProjectName, req.getProjectName())
                .like(StringUtils.isNotBlank(req.getBuildOrg()), ProjectCollection::getBuildOrg, req.getBuildOrg())
                .eq(Objects.nonNull(buildOrgCode), ProjectCollection::getBuildOrgCode, buildOrgCode);
        //处理 行政区域

        collectionService.page(page, wrapper);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }

        List<ProjectCollectionVO> res = page.getRecords().stream().map(w -> {
            ProjectCollectionVO vo = BeanUtil.copyProperties(w, ProjectCollectionVO.class);
            vo.setArea(regionCacheHelper.getDisplayName(w.getAreaCode(), RegionConst.RL_COUNTY));
            return vo;
        }).collect(Collectors.toList());

        return PageVo.of(res, page.getTotal());
    }

    /**
     * 项目归集详情
     *
     * @param id \
     * @return \
     */
    public ProjectCollectionVO detail(Long id) {
        ProjectCollection projectCollection = collectionService.getById(id);

        return BeanUtil.copyProperties(projectCollection, ProjectCollectionVO.class);
    }

    /**
     * 保存
     *
     * @param dto
     * @return
     */
    public String save(ProjectCollectionSaveDTO dto) {
        UserInfoDetails user = LoginUserUtil.userDetail();

        ProjectCollection projectCollection = BeanUtil.copyProperties(dto, ProjectCollection.class);
        if (Objects.nonNull(dto.getId())) {
            ProjectCollection old = collectionService.getById(dto.getId());
            VUtils.isTrue(Objects.isNull(old)).throwMessage("该项目归集不存在！");
            projectCollection.setId(dto.getId());
        } else {
            projectCollection.setProjectCode(projectCodeGenUtil.generateProjectCode(projectCollection));
            projectCollection.setCreateBy(user.getUsername());
            projectCollection.setCreateOn(LocalDateTime.now());
        }

        projectCollection.setUpdateBy(user.getUsername());
        projectCollection.setUpdateOn(LocalDateTime.now());
        if (collectionService.saveOrUpdate(projectCollection)) {
            return BizConst.SAVE_SUCCESS;
        }
        return BizConst.SAVE_FAIL;
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    public String delete(Long id) {
        ProjectCollection projectCollection = collectionService.getById(id);

        VUtils.isTrue(Objects.isNull(projectCollection)).throwMessage("该项目归集不存在 删除失败");

        if (collectionService.removeById(id)) {
            return "删除成功";
        }
        return "删除失败";
    }
}
