package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.util.NumberUtil;
import com.hz.pm.api.common.util.ThreadPoolUtil;
import com.hz.pm.api.projectdeclared.manage.*;
import com.hz.pm.api.projectdeclared.model.req.PurchaseAdaptionListReq;
import com.hz.pm.api.projectdeclared.model.vo.*;
import com.hz.pm.api.projectlib.model.req.DeclaredProjectListReq;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.AmountApprovalProgressVO;
import com.hz.pm.api.projectlib.model.vo.AnnualPlanProgressVO;
import com.hz.pm.api.projectlib.model.vo.DeclaredRecordProgressVO;
import com.hz.pm.api.projectlib.model.vo.ProjectManageTodoCountVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;

/**
 * <p>
 * ProjectTodoCountManage
 * </p>
 *
 * @author WendyYang
 * @since 10:51 2024/10/31
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectManageTodoCountManage {

    private final DeclaredProjectManage declaredProjectManage;
    private final AnnualPlanLibManage annualPlanLibManage;
    private final AmountApprovalManage amountApprovalManage;
    private final DeclaredRecordManage declaredRecordManage;
    private final PurchaseManage purchaseManage;
    private final ConstructionManage constructionManage;
    private final FinalAcceptanceManage finalAcceptanceManage;
    private final ProjectReviewManage projectReviewManage;

    public ProjectManageTodoCountVO allTodoCount(ProjectListReq req) {
        ProjectManageTodoCountVO ret = new ProjectManageTodoCountVO();
        List<CompletableFuture<Void>> futures = new ArrayList<>();
        Long buildOrgCode = NumberUtil.parseLong(req.getBuildOrgCode(), null);
        ProjectListReq projReq = ProjectListReq.builder()
                .projectYear(req.getProjectYear())
                .declaredUnitId(buildOrgCode)
                .build();
        ProjectListReq projReqOrg = ProjectListReq.builder()
                .projectYear(req.getProjectYear())
                .buildOrgCode(req.getBuildOrgCode())
                .build();
        futures.add(CompletableFuture.runAsync(() -> {
            // 项目申报
            DeclaredProgressVO declaredStat = declaredProjectManage.declaredProgress(projReq);
            ret.setProjectDeclare(sum(declaredStat, DeclaredProgressVO::getTodoCount,
                    DeclaredProgressVO::getFailedCount));
            // 专家评审
            ExpertReviewProgressVO expertReview = projectReviewManage.expertReviewProjectProgress(projReq);
            ret.setExpertReview(sum(expertReview, ExpertReviewProgressVO::getTodoCount));
            // 项目复核
            ExpertReviewProgressVO projectReview = projectReviewManage.reviewProjectProgress(projReq);
            ret.setProjectReview(sum(projectReview, ExpertReviewProgressVO::getTodoCount,
                    ExpertReviewProgressVO::getFailedCount));
        }, ThreadPoolUtil.REQUEST));
        futures.add(CompletableFuture.runAsync(() -> {
            // 年度计划
            AnnualPlanProgressVO annualPlan = annualPlanLibManage.annualPlanProgress(projReqOrg);
            ret.setAnnualPlan(sum(annualPlan, AnnualPlanProgressVO::getNotOpenAnnualPlanCount));
            // 资金下达
            projReqOrg.setBuildOrgCode(req.getBuildOrgCode());
            AmountApprovalProgressVO amountApproval = amountApprovalManage.amountApprovalProgress(projReqOrg);
            ret.setAmountApproval(sum(amountApproval, AmountApprovalProgressVO::getUnApprovalCount));
            // 立项备案
            DeclaredProjectListReq declaredReq = new DeclaredProjectListReq();
            declaredReq.setProjectYear(req.getProjectYear());
            declaredReq.setDeclaredUnitId(buildOrgCode);
            DeclaredRecordProgressVO declaredRecord = declaredRecordManage.declaredRecordProgress(declaredReq);
            ret.setProjectRecord(sum(declaredRecord, DeclaredRecordProgressVO::getTodoCount,
                    DeclaredRecordProgressVO::getFailedCount));
        }, ThreadPoolUtil.REQUEST));
        futures.add(CompletableFuture.runAsync(() -> {
            // 招标采购
            projReqOrg.setBuildOrgCode(req.getBuildOrgCode());
            PurchaseProgressVO purchase = purchaseManage.purchaseProgress(projReqOrg);
            ret.setProjectPurchase(sum(purchase, PurchaseProgressVO::getOnPurchaseCount, PurchaseProgressVO::getToBePurchaseCount));
            // 适配改造
            PurchaseAdaptionListReq adaptionReq = new PurchaseAdaptionListReq();
            adaptionReq.setBuildOrgCode(buildOrgCode);
            adaptionReq.setProjectYear(req.getProjectYear());
            AdaptionProgressVO adaptionStat = purchaseManage.adaptionProgress(adaptionReq);
            ret.setProjectAdaption(sum(adaptionStat, AdaptionProgressVO::getToBeAdapt, AdaptionProgressVO::getOnAdapting));
            // 项目初验
            projReqOrg.setBuildOrgCode(req.getBuildOrgCode());
            FirstAcceptProgressVO firstAccept = constructionManage.firstAcceptProgress(projReqOrg);
            ret.setFirstAccept(sum(firstAccept, FirstAcceptProgressVO::getTodoCount, FirstAcceptProgressVO::getDoingCount));
            // 项目终验
            projReqOrg.setBuildOrgCode(req.getBuildOrgCode());
            FinalAcceptProgressVO finalAccept = finalAcceptanceManage.finalAcceptProgress(projReqOrg);
            ret.setFinalAccept(sum(finalAccept, FinalAcceptProgressVO::getTodoCount,
                    FinalAcceptProgressVO::getFailedCount, FinalAcceptProgressVO::getXcfhxCount));
        }, ThreadPoolUtil.REQUEST));
        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0])).join();
        return ret;
    }

    @SafeVarargs
    private final <T> int sum(T obj, Function<T, Number>... function) {
        if (obj == null || function == null || function.length == 0) {
            return 0;
        }
        return Arrays.stream(function)
                .map(w -> w.apply(obj))
                .filter(Objects::nonNull)
                .mapToInt(Number::intValue)
                .sum();
    }

}
