package com.hz.pm.api.scheduler.listener;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.NumberUtil;
import com.google.common.collect.Sets;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.staging.enums.MsgTypeEnum;
import com.hz.pm.api.sys.manage.NoticeManage;
import com.hz.pm.api.todocenter.manage.HandlerManage;
import com.wflow.contants.HisProInsEndActId;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.process.enums.NodeTypeEnum;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.notify.event.ProcessEndEvent;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.HistoryService;
import org.flowable.engine.TaskService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.task.api.Task;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.Set;

import static com.hz.pm.api.todocenter.constant.WorkNoticeConst.PASS_MSG_TEMPLATE;
import static com.hz.pm.api.todocenter.constant.WorkNoticeConst.PASS_MSG_TEMPLATE2;

/**
 * 工作流开始 结束监听 自动审批的话 会漏掉 调用状态机
 *
 * @author ZPF
 * @return
 * @since 2023/04/14 14:19
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProcessEndListener {

    private final TaskService taskService;
    private final HistoryService historyService;
    private final IProjectInstService projectInstService;
    private final IProjectService projectService;
    private final ProcessInstanceService processInstanceService;
    private final HandlerManage handlerManage;
    private final NoticeManage noticeManage;

    @Value("${spring.profiles.active}")
    private String active;

    @Async
    @EventListener
    @Transactional(rollbackFor = Exception.class)
    public void onApplicationEvent(ProcessEndEvent event) {
        log.info("进入工作流结束的 事件监听!{}", event.getInstCode());
        String instCode = event.getInstCode();

        log.info("instCode:{}", instCode);

        //休息3秒
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }

        //1.去查找 项目实例关系表
        ProjectInst projectInst = projectInstService.getByInstCode(instCode);

        if (Objects.isNull(projectInst)) {
            return;
        }

        List<Task> tasks = taskService.createTaskQuery()
                .processInstanceId(instCode)
                .active()
                .list();

        List<HistoricTaskInstance> historyTasks = historyService.createHistoricTaskInstanceQuery()
                .processInstanceId(instCode)
                .unfinished()
                .list();

        //已经没有任务了
        if (CollUtil.isEmpty(tasks) && CollUtil.isEmpty(historyTasks)) {
            Long projectId = projectInst.getProjectId();
            Project project = projectService.getNewProject(projectId);
            HistoricProcessInstance instance = historyService.createHistoricProcessInstanceQuery()
                    .processInstanceId(instCode).singleResult();
            //通过审核后 所处理的逻辑
            afterPassTodo(project, instance);
        }
    }

    /**
     * 审核通过后 所处理的逻辑
     *
     * @param project  \
     * @param instance \
     */
    private void afterPassTodo(Project project, HistoricProcessInstance instance) {
        // 获取流程通过后的流程实例
        HistoricProcessInstance newInstance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(instance.getId())
                .singleResult();
        // 获取当前流程实例类型
        String instanceId = newInstance.getId();
        ProjectInst projectInst = projectInstService.getByInstCode(instanceId);
        Integer instType = projectInst.getInstType();
        InstTypeEnum instTypeEnum = InstTypeEnum.getNoNull(instType);
        // 获取流程通过后当前流程详情
        ProcessProgressVo newInstDetail = processInstanceService.getProgressInstanceDetail(null, instance.getId());
        // 获取流程通过后当前审核人信息，向其发送工作通知
        // 会签/或签会有多个审核人
        Set<String> currUserIdList = Sets.newHashSet();
        List<ProgressNode> newProgressInfo = newInstDetail.getProgressInfo();
        ProgressNode currNode = newProgressInfo.get(newProgressInfo.size() - 1);

        // 说明当前节点是子流程节点
        if (currNode.getNodeType().name().equals(NodeTypeEnum.SUB.name())) {
            List<ProgressNode> children = currNode.getChildren();
            // 获取子流程当前审核人节点
            ProgressNode subCurrNode = CollUtil.isEmpty(children) ? new ProgressNode() : children.get(children.size() - 1);
            // 获取节点ID相同地审核节点
            children.stream().filter(c -> subCurrNode.getNodeId().equals(c.getNodeId()))
                    .forEach(c -> currUserIdList.add(c.getUserId()));
        } else {
            newProgressInfo.stream().filter(c -> currNode.getNodeId().equals(c.getNodeId()))
                    .forEach(c -> currUserIdList.add(c.getUserId()));
        }

        // 流程通过后，判断当前登录用户是不是最后一个审核人
        // 若当前登录用户是最后一个审批人，需更新流程状态为审核完成，项目状态到下个状态
        // 并向流程发起人发送浙政钉工作通知：【项目名称】已通过【流程名称】，请及时开始下一步操作。
        if (HisProInsEndActId.END.equals(newInstance.getEndActivityId())) {
            //发送消息
            noticeManage.sendNotice(Long.parseLong(instance.getStartUserId()), project, instance.getProcessDefinitionName(),
                    PASS_MSG_TEMPLATE2, MsgTypeEnum.PROJECT_REVIEW_PASS);
        } else {
            // 若有下一个审核人(当前节点的用户)，会签/或签会有多个
            // 向其发送浙政钉工作通知：标题：审核任务 内容：【单位名称】的【项目名称】需要您审核。
            // 获取发送浙政钉工作通知必要信息
            for (String currUserId : currUserIdList) {
                if (NumberUtil.isNumber(currUserId)) {
                    //发送消息
                    noticeManage.sendNotice(Long.parseLong(currUserId), project, instance.getProcessDefinitionName(),
                            PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);
                } else {
                    log.warn("无效的审核人用户Id：{}", currUserId);
                }

            }
        }
    }
}