package com.hz.pm.api.scheduler.task;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.external.MhApiClient;
import com.hz.pm.api.external.model.dto.MhZwddWorkNoticeDTO;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.external.todo.dto.MhTodoExtraParamDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.service.IMhProjectService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.model.entity.MhTodoRecord;
import com.hz.pm.api.sys.service.IMhTodoRecordService;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.model.enumeration.RoleEnum;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.ningdatech.basic.util.CollUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import static com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum.DECLARED_RECORD_APPLY;

/**
 * <p>
 * DeclaredRecordReminderTask
 * </p>
 *
 * @author WendyYang
 * @since 10:40 2024/3/14
 */
@Slf4j
@Component
public class DeclaredRecordReminderTask extends AbstractReminderTask {

    private final IProjectService projectService;
    private final IMhTodoRecordService todoRecordService;
    private final MhApiClient mhApiClient;
    private final MHTodoClient mhTodoClient;
    private final MhUnitCache mhUnitCache;
    private final UserInfoHelper userInfoHelper;
    private final IMhProjectService mhProjectService;


    public DeclaredRecordReminderTask(Environment environment,
                                      IProjectService projectService,
                                      IMhTodoRecordService todoRecordService,
                                      MhApiClient mhApiClient,
                                      MHTodoClient mhTodoClient,
                                      MhUnitCache mhUnitCache,
                                      UserInfoHelper userInfoHelper,
                                      IMhProjectService mhProjectService) {
        super(environment);
        this.projectService = projectService;
        this.todoRecordService = todoRecordService;
        this.mhApiClient = mhApiClient;
        this.mhTodoClient = mhTodoClient;
        this.mhUnitCache = mhUnitCache;
        this.userInfoHelper = userInfoHelper;
        this.mhProjectService = mhProjectService;
    }

    @Override
    public String getTaskPropertyName() {
        return "declared-record";
    }

    private static final String MSG_FMT = "【%s】需要发起立项备案申请，请及时处理。";

    @Override
    public void execute() {
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(Project::getStatus, ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.getCode());
        List<Project> projects = projectService.list(query);
        if (projects.isEmpty()) {
            return;
        }
        List<String> projectCodes = CollUtils.fieldList(projects, Project::getProjectCode);
        List<Long> buildOrgCodes = projects.stream()
                .map(Project::getBuildOrgCode)
                .filter(w -> StrUtil.isNotBlank(w) && NumberUtil.isNumber(w))
                .map(NumberUtil::parseLong)
                .collect(Collectors.toList());
        if (buildOrgCodes.isEmpty()) {
            return;
        }
        List<UnitDTO> units = mhUnitCache.listByIds(buildOrgCodes);
        Map<String, UnitDTO> unitMap = CollUtils.listToMap(units, w -> String.valueOf(w.getId()));
        List<MhTodoRecord> todoRecords = todoRecordService.list(DECLARED_RECORD_APPLY, projectCodes);
        Map<String, MhTodoRecord> todoRecordMap = CollUtils.listToMap(todoRecords, MhTodoRecord::getBizId);
        List<UserFullInfoDTO> users = userInfoHelper.listUserFullInfoByUnitIdsAndRole(buildOrgCodes, RoleEnum.COMPANY_MANAGER);
        if (users.isEmpty()) {
            return;
        }
        Map<String, List<UserFullInfoDTO>> userMap = CollUtils.group(users, UserFullInfoDTO::getMhUnitIdStr);
        for (Project project : projects) {
            try {
                String projectCode = project.getProjectCode();
                Optional<Long> mhProjectId = mhProjectService.getIdByProjectCode(projectCode);
                if (mhProjectId.isPresent()) {
                    String buildOrgCode = project.getBuildOrgCode();
                    if (unitMap.containsKey(buildOrgCode) && userMap.containsKey(buildOrgCode)) {
                        List<UserFullInfoDTO> currUnitUsers = userMap.get(buildOrgCode);
                        if (currUnitUsers == null) {
                            continue;
                        }
                        sendNoticeToAuditor(project, currUnitUsers, todoRecordMap, projectCode, mhProjectId);
                    }
                }
            } catch (Exception e) {
                log.error("通知失败", e);
            }
        }
    }

    private void sendNoticeToAuditor(Project project, List<UserFullInfoDTO> currUnitUsers, Map<String, MhTodoRecord> todoRecordMap, String projectCode, Optional<Long> mhProjectId) {
        String content = String.format(MSG_FMT, project.getProjectName());
        for (UserFullInfoDTO user : currUnitUsers) {
            try {
                if (todoRecordMap.containsKey(project.getProjectCode())) {
                    MhZwddWorkNoticeDTO notice = MhZwddWorkNoticeDTO.builder()
                            .title(DECLARED_RECORD_APPLY.getVal())
                            .content(content)
                            .targetUserName(user.getRealName())
                            .targetUser(user.getMhUserId())
                            .build();
                    mhApiClient.sendZwddWorkNotice(notice);
                } else {
                    MhTodoExtraParamDTO paramObj = MhTodoExtraParamDTO.builder()
                            .projectId(project.getId())
                            .projectCode(projectCode)
                            .mhProjectId(mhProjectId.get())
                            .status(project.getStatus())
                            .path("/declareManage/projectFilingEdit")
                            .userId(user.getUserId())
                            .userName(user.getRealName())
                            .build();
                    mhTodoClient.addTodo(paramObj, user, DECLARED_RECORD_APPLY, projectCode, content);
                }
            } catch (Exception e) {
                log.error("立项备案提醒发送失败：{}", project.getProjectCode(), e);
            }
        }
    }

}
